/*
 * A simple CAMAC shell
 * for use with my (ole) CAMAC driver
 */

/* changes by chw */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <camac.h>

#ifdef USE_READLINE
#include <readline/readline.h>
#include <readline/history.h>
#else /* !USE_READLINE */
char *readline(char *prompt) /* simulate Gnu readline() */
{
  char *s = (char *)malloc(1024);

  fputs(prompt,stdout);
  fflush(stdout);
  fgets(s,1024,stdin);
  return s;
}

void add_history(char *s) {} /* simulate history() */
#endif /* !USE_READLINE */

void printhelp(void); /* print out some help */

int main(int argc, char **argv)
{

  char prompt[20];                               /* hold the cmcsh prompt */
  char *s;                                                  /* input line */
  char devicepath[80];              /* full file path of the CAMAC device */
  char device[80];                       /* file name of the CAMAC device */

  int crate_fd;                          /* file descriptor for the crate */
  int crate_nr = 0;                                       /* crate number */
  int n = 1,a = 0,f = 0;             /* position, subaddress and function */
  int t = 100;                                /* timeout in 10 msec units */
  int qx = 0;                                             /* Q and X flag */
  long data;                                         /* input/output data */
  int tmp;          /* temporary variable, for token search of input line */
  int tmp_fd;       /* temporary file descriptor, for opening a new crate */
  char tmp_dev[80];     /* temporary device name for opening a new device */

  printf("CAMAC shell\n");

  if (argc < 2)                               /* if no cmd line arguments */
    for (crate_nr = 0; crate_nr < 15; crate_nr++) {   /* search the first */
      sprintf(device,"camac%i",crate_nr);                 /* usable crate */
      sprintf(devicepath,"/dev/%s",device);
      if ((crate_fd = open(devicepath,O_RDWR)) > 0) break;
    } /* for i */
  else {
    if (isdigit(*argv[1]))          /* first argument is the crate number */
      sprintf(device,"camac%i",atoi(argv[1]));
    else                                                /* or device name */
      sprintf(device,"%s",argv[1]);
    sprintf(devicepath,"/dev/%s",device);
    if ((crate_fd = open(devicepath,O_RDWR)) <= 0) {
      perror(devicepath);
      exit(1);
    }

  } /* if argc < 2 */
  
  if (crate_nr == 15) {
    printf("cmcsh: error: no crate found\n");
    exit(1);
  }

/* now start the loop on the command line */  
  sprintf(prompt,"%s > ", device);
  while((s=readline(prompt))) {
    int line_error=0;                /* detects if there is a parsing error */
    char line_delimiter[]=" ,;";            /* delimiter between the inputs */
    char *token_pt=NULL;                       /* pointer to the next token */
    char *s_copy;                               /* copy of the input string */
    size_t length;                              /* input string length (+1) */

    length = strlen(s) + 1;
    s_copy = (char *) malloc(length);
    strcpy(s_copy,s);

    token_pt=strtok(s_copy,line_delimiter);            /* search next token */
    if (!token_pt){                                         /* if not found */
      line_error++;
      free(s_copy);
    }
    else{                                               /* if a token found */
      do {                                  /* while there is a valid token */
	switch (token_pt[0]) {
	case 'n':                                           /* set position */
	  if (sscanf(token_pt,"n=%i",&tmp) == 1)
	    n = tmp;
	  else{
	    line_error++;
	    printf("Error reading Address n (%s)\n",token_pt);
	  }
	  break;
	case 'a':                                         /* set subaddress */
	  if (sscanf(token_pt,"a=%i",&tmp) == 1)
	    a = tmp;
	  else{
	    line_error++;
	    printf("Error reading Subaddress a (%s)\n",token_pt);
	  }
	  break;
	case 'f':                                            /* set function */
	  if (sscanf(token_pt,"f=%i",&tmp) == 1)
	    f = tmp;
	  else{
	    line_error++;
	    printf("Error reading Function f (%s)\n",token_pt);
	  }
	  break;
	case 'd':                                               /* set data */
	  if (sscanf(token_pt,"d=%i",&tmp) == 1)
	    data = tmp;
	  else{
	    line_error++;
	    printf("Error reading Data d (%s)\n",token_pt);
	  }
	  break;
	case 'l':                                         /* wait for a LAM */
	  if (sscanf(token_pt,"l=%i",&tmp) == 1) 
	    t = (tmp+5)/10;
	  ioctl(crate_fd, CAMAC_WAIT_LAM, &t);
	  break;
	case 'c':                                          /* set new crate */
	  if (sscanf(token_pt,"c=%s",tmp_dev) == 1) {
	    if (isdigit(*tmp_dev))
	      sprintf(tmp_dev,"camac%i",atoi(tmp_dev));
	    if (strcmp(tmp_dev,device) == 0) 
	      break;
	    sprintf(devicepath,"/dev/%s",tmp_dev);
	    if ((tmp_fd = open(devicepath,O_RDWR)) > 0) {
	      close(crate_fd);
	      crate_nr = tmp;
	      crate_fd = tmp_fd;
	      sprintf(device, "%s", tmp_dev);
	      sprintf(prompt, "%s > ", device);
	    }
	    else 
	      perror(devicepath);
	  } /* if sscanf */
	  else{
	    line_error++;
	    printf("Error reading Crate number c (%s)\n",token_pt);
	  }
	  break;      
	case 'e':                                         /* execute command */
	  CNAF24(crate_fd,n,a,f,&data);
	  ioctl(crate_fd,CAMAC_STATUS,&qx);
	case 'p':                                         /* print parameter */
	  printf("n=%i a=%i f=%i d=%li q=%i x=%i\n",n,a,f,data,
		 CAMAC_Q_FLAG(qx), CAMAC_X_FLAG(qx));
	  break;
	case 'q':                                                    /* quit */
	  line_error++;
	  exit(0);
	default:                                     /* unrecognized command */
	  line_error++;
	  printhelp();
	} /* switch */
      } while ((token_pt = strtok(NULL,line_delimiter))); /* do */
      free(s_copy);
    } /* else */

/* add to the history buffer if no error occures */
    if (!line_error) {
      add_history(s);
    }

    free(s);
  } /* while readline */

  exit(2); /* never reached */

} /* main() */

void printhelp(void)
{
  printf("Syntax:\n");
  printf("n=12 ..... set address to 12\n");
  printf("a=3 ...... set subaddress to 3\n");
  printf("f=1 ...... set function code 1\n");
  printf("l=1000 ... wait 1000 msec for a LAM\n");
  printf("c=1 ...... set CAMAC crate\n");
  printf("p ........ print parameters\n");
  printf("e ........ execute CAMAC cycle\n");
  printf("q ........ quit\n");
}

